require "claide"
require "colored"

module Dod
  class Runner < CLAide::Command

    self.summary = "Creates Definition of Done task list for pull request linked to current branch and remote."
    self.command = "dod"
    self.version = Dod::VERSION

    def initialize(argv)
      donefile = argv.option("donefile", "Donefile")
      @donefile_path = donefile if File.exist?(donefile)
      @repository = Repository.new
      @donefile = Donefile.new
      super
    end

    def validate!
      super
      if self.class == Runner && !@donefile_path
        help!("Could not find a Donefile.")
      end
    end

    def self.options
      [
        ["--donefile=<path/to/donefile>", "Path to the Donefile where all tasks for Definition of Done are defined."]
      ]
    end

    def run
      tasks = @donefile.parse_tasks(@donefile_path)
      project_name = @repository.project_name(ENV)
      repo_name = @repository.repo_name(ENV)
      current_branch = ENV["bamboo_repository_git_branch"]
      unless project_name && repo_name && current_branch
        puts "Lack of definition of one of the environment variable (bamboo_repository_git_branch or bamboo_planRepository_repositoryUrl).".red
        return
      end
      bitbucket = BitbucketServerAPI.new(ENV, project_name, repo_name, current_branch)
      bitbucket.create_definition_of_done(tasks)
    end
  end
end
